package com.displaynone.acss.ui.init

import android.content.Context
import android.net.ConnectivityManager
import android.net.NetworkCapabilities
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.NavController
import androidx.navigation.fragment.findNavController
import com.displaynone.acss.R
import com.displaynone.acss.components.auth.models.user.UserServiceST
import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO
import com.displaynone.acss.databinding.FragmentInitBinding
import com.displaynone.acss.util.collectWithLifecycle
import com.displaynone.acss.util.navigateTo

class InitFragment : Fragment(R.layout.fragment_init) {
    private var _binding: FragmentInitBinding? = null
    private val binding: FragmentInitBinding get() = _binding!!

    private val viewModel: InitFragmentViewModel by viewModels()


    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentInitBinding.bind(view)

        if (!checkInternetConnection()) {
            handleError(R.string.noInternet)
            return
        }

        pingServer()

    }

    private fun pingServer() {
        viewModel.pingServer()
        viewModel.state.collectWithLifecycle(this) { state ->
            if (state is InitFragmentViewModel.State.Show){
                if(!state.item){
                    handleError(R.string.serverIsUnabailable)
                } else{
                    val navController: NavController = findNavController()

                    if (!isUserAuthenticated()) {
                        navigateTo(navController, R.id.action_nav_init_to_nav_auth)
                    } else {
                        updateUser(navController)
                    }
                }
            }
        }
    }

    private fun updateUser(navController: NavController) {
        viewModel.updateUserInfo()
        viewModel.state.collectWithLifecycle(this){ state ->
            if (state is InitFragmentViewModel.State.Update) {
                if (state.item == null) {
                    navigateTo(navController, R.id.action_nav_init_to_nav_auth)
                } else{
                    navigateTo(navController, R.id.action_nav_init_to_nav_profile)
                }
            }
            if (state is InitFragmentViewModel.State.Error){
                navigateTo(navController, R.id.action_nav_init_to_nav_auth)
            }
        }
    }

    private fun checkInternetConnection(): Boolean {
        if (context == null) return false;

        val connectivityManager =
            context?.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val capabilities =
            connectivityManager.getNetworkCapabilities(connectivityManager.activeNetwork)
                ?: return false

        if (capabilities.hasTransport(NetworkCapabilities.TRANSPORT_CELLULAR)) {
            Log.i("Internet", "NetworkCapabilities.TRANSPORT_CELLULAR")
            return true
        } else if (capabilities.hasTransport(NetworkCapabilities.TRANSPORT_WIFI)) {
            Log.i("Internet", "NetworkCapabilities.TRANSPORT_WIFI")
            return true
        } else if (capabilities.hasTransport(NetworkCapabilities.TRANSPORT_ETHERNET)) {
            Log.i("Internet", "NetworkCapabilities.TRANSPORT_ETHERNET")
            return true
        }

        return false
    }

    private fun getCachedUser(): UserDTO? {
        return UserServiceST.getInstance().getUserDTO();
    }

    private fun isUserAuthenticated(): Boolean {
        return UserServiceST.getInstance().hasTokens()
    }

    private fun handleError(string: Int) {
        binding.error.text = requireContext().getString(string)
    }
    private fun handleError(errorMessage: String) {
        binding.error.text = errorMessage
    }
}